<?php

namespace Bixo\Download\Http\Controllers;

use App\Http\Controllers\ActionController as BaseController;
use Exception;
use Bixo\Download\Actions\DownloadAction;
use Bixo\Download\Http\Requests\DownloadActionRequest;
use Bixo\Download\Http\Resources\DownloadResource;
use Bixo\Download\Models\Download;

/**
 * Admin web controller class.
 */
class DownloadActionController extends BaseController
{

    /**
     * Action controller function for download.
     *
     * @param Model $model
     * @param action next action for the model.
     *
     * @return Response
     */
    public function __invoke(DownloadActionRequest $request, Download $download, $action)
    {
        try {
            $request = $request->all();
            $download = DownloadAction::run($action, $download, $request);

            $data = new DownloadResource($download);
            return $this->response->message(trans("messages.success.{$action}", ['Module' => trans('download::download.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('download/download/' . $download->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('download/download/' . $download->getRouteKey()))
                ->redirect();
        }
    }

    public function notify(){
        
    }
}
