<?php

namespace Bixo\Download\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Bixo\Download\Events\DownloadAction as DownloadActionEvent;
use Bixo\Download\Notifications\DownloadAction as DownloadActionNotification;
use Litepie\Actions\Concerns\AsAction;

class DownloadAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the DownloadActionEvent.
     *
     * @param   DownloadActionEvent  $event
     * @return mixed
     */
    public function handle(DownloadActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $download.
     *
     * @param   DownloadActionEvent  $event
     * @return void
     */
    public function create(DownloadActionEvent $event)
    {
        $client = $event->download->client;
        Notification::send($client, new DownloadActionNotification($event));
    }

    /**
     * Handle the DownloadActionEvent as a listener.
     *
     * @param   DownloadActionEvent  $event
     * @return mixed
     */
    public function asListener(DownloadActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   DownloadActionEvent  $event
     * @return bool
     */
    private function isAllowed(DownloadActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
