<?php
namespace Bixo\Download\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Bixo\Download\Events\DownloadAction as DownloadActionEvent;
use Bixo\Download\Models\Download;

class DownloadAction extends Notification implements ShouldQueue
{
    use Queueable;

    protected $event;
    protected $download;
    protected $action;
    protected $request;

    /**
     * Create a new notification instance.
     *
     * @param DownloadActionEvent $download
     */
    public function __construct(DownloadActionEvent $download)
    {
        $this->action = $event->action;
        $this->download = $download;
        $this->request = $event->request;
        $this->event = $event;

    }

    /**
     * Get the mail representation of the notification.
     *
     * @param mixed $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->subject('Download Approved - ' . config('app.name'))
            ->view('download::download.email.action', ['data' => $this->download]);
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param object $notifiable
     * @return array<int, string>
     */
    public function via(object $notifiable): array
    {
        return ['mail'];
    }
}
