<?php

namespace Bixo\Download\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Bixo\Download\Models\Download;

class DownloadPolicy
{

    use DownloadAction;
    use DownloadReport;

    /**
     * Determine if the given user can view the download.
     *
     * @param Authenticatable $user
     * @param Download $download
     *
     * @return bool
     */
    public function view(Authenticatable $user, Download $download)
    {
        if ($authUser->canDo('download.download.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $download->is_owner;
    }

    /**
     * Determine if the given user can create a download.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('download.download.create');
    }

    /**
     * Determine if the given user can update the given download.
     *
     * @param Authenticatable $user
     * @param Download $download
     *
     * @return bool
     */
    public function update(Authenticatable $user, Download $download)
    {
        if ($user->canDo('download.download.edit') && $user->isAdmin()) {
            return true;
        }

        return $download->is_owner;
    }

    /**
     * Determine if the given user can delete the given download.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Download $download)
    {
        return $download->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
